library kostasoft_mjpeg;

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;

import 'dart:typed_data';
import 'dart:async';

class MjpegView extends StatefulWidget {
  final String url;
  final bool isFullscreen;
  MjpegView({this.url, this.isFullscreen = false});

  @override
  MjpegViewState createState() => MjpegViewState();
}

class MjpegViewState extends State<MjpegView> {
  http.Client client = http.Client();
  StreamSubscription videoStream;
  StreamController _streamController = StreamController<Widget>();

  @override
  void initState() {
    super.initState();
    start();
  }

  start() async {
    await Future.delayed(Duration.zero);
    var request = http.Request("GET", Uri.parse(widget.url));
    client.send(request).then((response) {
      var isInside = false;
      List<int> buf = List<int>();
      var old = 0x00;
      videoStream = response.stream.listen((List<int> data) {
        double height = MediaQuery.of(context).size.height;
        double width = MediaQuery.of(context).size.width;
        bool isLandscape = width > height;
        for (var i = 0; i < data.length; i++) {
          var b = data[i];
          if (isInside) {
            buf.add(b);
            if (b == 0xd9 && old == 0xff) {
              isInside = false;
              Image mjpeg = Image.memory(Uint8List.fromList(buf),
                fit: widget.isFullscreen ? (isLandscape ? BoxFit.fitWidth : BoxFit.fitHeight) : (isLandscape ? BoxFit.fitHeight : BoxFit.fitWidth),
                alignment: Alignment.center,
                height: height,
                width: width,
              );
              precacheImage(mjpeg.image, context).then((_) {
                _streamController.add(mjpeg);
              });
              buf = List<int>();
            }
          } else if (b == 0xd8 && old == 0xff) {
            buf = [0xff, 0xd8];
            isInside = true;
          }
          old = b;
        }
      });
    }).catchError((error) {
      _streamController.add(Center(
          child: Padding(padding: EdgeInsets.all(16.0),child: Text(error.toString(), textAlign: TextAlign.center, style: TextStyle(color: Colors.red)))));
    });
  }

  @override
  void deactivate() {
    videoStream?.cancel();
    client?.close();
    super.deactivate();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder(
        stream: _streamController.stream,
        builder: (context, snapshot) {
          if (!(snapshot?.hasData ?? false)) {
            return  Center(child: CircularProgressIndicator());
          } else if (snapshot.hasError) {
            return snapshot.data;
          } else {
            return snapshot.data;
          }
        });
  }
}
